namespace eval cdt_script {proc processKeyValue {key value verbose props fl} {
	if {$verbose == 0} {
		global cdt_script::$key
		set $key $value
	} 
	if {$verbose >= 1} {
		puts $fl "$key: $value"
	} 
	if {$verbose == 2} {
		set foo [$props setProperty $key $value]
	}
}

set EntryKinds {
	CDT_LIBRARY
	CDT_PROJECT
	CDT_SOURCE
	CDT_INCLUDE
	CDT_CONTAINER
	CDT_MACRO
	CDT_OUTPUT
	CDT_INCLUDE_FILE
	CDT_MACRO_FILE
}

proc getNameAndLocation {props fl project verbose} {
	# Basic org.eclipse.core.resources.project attributes (not specific to CDT project, but to eclipse project)
	if {$verbose >= 1} {
		puts $fl "======================="
		puts $fl "===   CDT PROJECT   ==="
		puts $fl "======================="
		puts $fl "-- Name and Location --"
	}
	
	set curStr [eclog::getDirectory]
	processKeyValue "root" $curStr $verbose $props $fl 
	set curStr [$project getName]
	processKeyValue "name" $curStr $verbose $props $fl 
}

proc getIncludes {props fl includes verbose} {
	set includesCount [$includes length] 	
	processKeyValue "includesCount" $includesCount $verbose $props $fl 
	set includesList ""

	set i 0
	java::for {org.eclipse.cdt.core.model.IIncludeReference curInclude} $includes {
		set curStr [[[$curInclude getIncludeEntry] getFullIncludePath] toString]
		processKeyValue "include$i" $curStr $verbose $props $fl 
		append includesList "-I $curStr "
		incr i
	}
	processKeyValue "includesList" $includesList $verbose $props $fl 
}

proc getMacros {props fl paths verbose} {	
	set macrosList ""
	set macrosString ""
	
	set i 0
	java::for {org.eclipse.cdt.core.model.IPathEntry curEntry} $paths {
		set curStr [$curEntry getEntryKind]
		if {$curStr == [java::field org.eclipse.cdt.core.model.IPathEntry CDT_MACRO]} {
			append macrosList "macro$i,"
			set contents [$curEntry toString]
			set macroName ""
			set macroValue ""
			set result [regexp {.* name:(.*) value:(.*)} $contents matched macroName macroValue]
			if {$result==0} {
				set result [regexp {.* name:(.*)} $contents matched macroName]
			}
			if {$macroValue != ""} {
				append macroName "=$macroValue"
			}
			append macrosString "-D$macroName "
			processKeyValue "macro$i" $macroName $verbose $props $fl 
			processKeyValue "macro$i.name" $macroName $verbose $props $fl 
			processKeyValue "macro$i.value" $macroValue $verbose $props $fl 
			incr i
		   } 
		   
	}
	processKeyValue "macrosList" $macrosList $verbose $props $fl 
	processKeyValue "macrosString" $macrosString $verbose $props $fl 
	processKeyValue "macrosCount" $i $verbose $props $fl 
}

proc getSources {props fl project verbose} {
	set members [$project members]
	set sourcesList ""
	set sourcesString ""

	set i 0
	java::for {org.eclipse.core.resources.IResource curResource} $members {
		set curInt [$curResource getType]
		set curInteger [java::new Integer $curInt]
		set curType [$curInteger toString]

		set curExt [$curResource getFileExtension]
		
		if {$curExt != "" && $curExt != "logiscope_rchk" && $curExt != "logiscope_rule" && $curExt != "settings" && $curExt != "cdtproject" && $curExt != "project" && $curExt != "rst" && $curExt != "ttp" && $curExt != "ttw" && $curExt != "html"} {
			append sourcesList "source$i,"
			set curStr [$curResource getName]
			processKeyValue "source$i.name" $curStr $verbose $props $fl 
			append sourcesString "\"$curStr\" "
			set curStr [$curResource getFileExtension]
			processKeyValue "source$i.fileExt" $curStr $verbose $props $fl 
			set curStr [[$curResource getFullPath] toString]
			processKeyValue "source$i.fullPath" $curStr $verbose $props $fl 
			set curStr [[$curResource getLocation] toString]
			processKeyValue "source$i.location" $curStr $verbose $props $fl 
			incr i
		}
	}
	set sourcesCount $i 
	processKeyValue "sourcesCount" $sourcesCount $verbose $props $fl 
	processKeyValue "sourcesList" $sourcesList $verbose $props $fl 
	processKeyValue "sourcesString" $sourcesString $verbose $props $fl 
}

# This methods scans the project, and produces all parameters used by the Logiscope Plugin.
# Note that the verbose variable (default value = 0) is used to produce text files containing the project properties.
# - If verbose = 0, no file is generated
# - If verbose = 1, a plain text file detailing the parameters is produced
# - If verbose = 2, the plain text file is produced, and another XML properties files is also generated.
# Keep in mind that due to the Map-like structure of the java.util.Properties class, the properties file is NOT ORDERED.
#
# @param: project This is the project in which all parameters are looked for.
# @result: There is no actual returned result, but the following variables are created.
# root: The absolute path of the project.
# name: The project name.
# includesCount: The number of includes found in the project.
# include<i>: The absolute path for the current include. <i> ranges from 0 to includesCount-1.
# macrosString: The string of all macros (separator = whitespace).
# macrosCount: The number of macros found in the project.
# macro<i>.name: The current macro name. <i> ranges from 0 to macrosCount-1.
# macro<i>.value: The current macro value (can be empty). <i> ranges from 0 to macrosCount-1.
# sourcesString: The string of all source files absolute paths (separator = whitespace).
# sourcesCount: The number of source files found in the project.
# source<i>.name: The current source file name (<fileName>.<extension>). <i> ranges from 0 to sourcesCount-1.
# source<i>.fileExt: The current source file extension. <i> ranges from 0 to sourcesCount-1.
# source<i>.fullPath: The current source file relative path in the project. <i> ranges from 0 to sourcesCount-1.
# source<i>.location: The current source file absolute path. <i> ranges from 0 to sourcesCount-1.
proc getCParams {project} {
	set verbose 0
	set fl ""
	set props ""

	if {$verbose >= 1} {
		set fl [open "CDTProperties.txt" w]
	}
	if {$verbose == 2} {
		set propFileName "CDTProperties.xml"
		set props [java::new java.util.Properties]
	}

	# A CDT project is a org.eclipse.cdt.core.model.ICProject
	set core [java::call org.eclipse.cdt.core.model.CoreModel getDefault]
	set newProject [$core create $project]

	# Retrieve name and location variables
	getNameAndLocation $props $fl $project $verbose

	# Retrieve includes
	set includes [$newProject getIncludeReferences]    
	getIncludes $props $fl $includes $verbose

	# Retrieve macro definitions
	set paths [$newProject getResolvedPathEntries]
	getMacros $props $fl $paths $verbose

	# Retrieve source files
	set sources [$newProject getAllSourceRoots]
	getSources $props $fl $project $verbose

	if {$verbose >= 1} {
		close $fl
	}
	if {$verbose == 2} {
		set propsFile [java::new java.io.FileOutputStream $propFileName]
		set foo [$props store $propsFile "Properties File"]
		set foo [$propsFile close]
	}
}
}