# This script is sourced when the user tries to build
# a logiscope RuleChecker project

source [file join $::CommonDirectory misc.tcl]
source [file join $::EnvironmentDirectory env_log.tcl]
source [file join $::GuiDirectory cmn_gui.tcl]

# Name: create_rule
# Parameter: $project the java selected project from Eclipse platform
#            $lang the language of the current project
# Return: None
#
# This proc is in charge of setting specific ant properties for Logiscope
# rule checker projects.
# The proc starts by building a bridge ruleset file (bridge.rst) to make the
# link between settings from Eclipse and Logiscope.
proc create_rule {project language} {
    # Make the bridge rst file
    set bridge [file join [eclog::getTtpDirectory] $::BridgeFile]
    build_rst $language $bridge

    # Inform the ant build about this rst file
    eclog::addProperty rst $bridge

    # Inform the ant build about the output directory to use
    eclog::addProperty o_dir [eclog::getReportDirectory]
    
    # Inform the ant build about relaxation mechanism
    if {[eclog::getRelaxationState]} {
        eclog::addProperty relax "-relax"
    } else {
        eclog::addProperty relax ""
    }
    # Inform the ant build about use of frame for reports
    if {[eclog::getFrameState]} {
        eclog::addProperty frame ""
    } else {
        eclog::addProperty frame "-noframe"
    }
    
    # Tries if something special concerning the language
    create_rule_$language $project
}

proc create_rule_c {project} {
    if {![llength $::SourceExtensions]} {
    	set $::SourceExtensions ".c"
	}
	eclog::addProperty sources *[join [split $::SourceExtensions] \;*]
}

proc create_rule_c++ {project} {
    if {![llength $::SourceExtensions]} {
    	set $::SourceExtensions ".C"
	}
	eclog::addProperty sources *[join [split $::SourceExtensions] \;*]
    if {![llength $::HeaderExtensions]} {
    	set $::HeaderExtensions ".C"
	}
	eclog::addProperty headers *[join [split $::HeaderExtensions] \;*]
}

proc create_rule_java {project} {
}

proc create_rule_ada {project} {
    output "NOT IMPLEMENTED !"
}

# Name: build_rst
# Parameter: $language the language of the current project
#	     $rst the name of the rst file to generate
# Return: None
#
# Dupplicated code from wizard.tcl
proc build_rst {language rst} {
    env_read::init $language
    cmn_gui::setCurrentCfg $rst 1
    cmn_gui::computeRuleSets
    cmn_gui::setProjectRuleSet [eclog::getRuleSets]

    foreach mnemo [cmn_gui::getRules] {
	set key [env_elt::get_rule_key $mnemo]
	set st [eclog::getRuleState $mnemo]
	# Error and warning are the same for Logiscope
	if {$st == 2} { set st 1}
	if {[expr $st] != [expr [env_elt::get_state $key]]} {
	    set key [env_elt::get_or_duplicate $key]
	    env_elt::set_state $key $st
	}
    }

    # Create directories if needed
    file mkdir [eclog::getTtpDirectory]	
    
    # open and go for file generation
    set f [open $rst w]
    puts $f "METRICS VERSION 3\n"
    puts $f "/* Generated for project as a bridge between Eclipse and Logiscope */\n"
    # In first, writes the information about inlcude
    foreach path [env_cfg::get_includes $rst] {
	if {[env_read::is_standard_cfg $path]} {
	    puts $f "/* Found in directory [file dirname $path] */"
	    puts $f "INCLUDE \"[file tail $path]\" END INCLUDE\n"
	} else {
	    # The included config file is not in the Logiscope environment
	    # write down its directory information
	    puts $f "INCLUDE \"[file tail $path]\" IN \"[file dirname $path]\" END INCLUDE\n"
	}
    }

    # Writes in the file only rules which state is not the inherited one
    set header "\n/* Redefinition of STANDARDS status */\n"
    foreach mnemo [env_elt::get_rules_mnemo $rst] {
	set key [env_elt::get_rule_key $mnemo]
	if {[env_elt::get_state $key]} {
	    set status ON
	} else {
	    set status OFF
	}
	set dir [env_elt::get_directory $key]
	if {[string length $dir]} {
	    set dir " IN \"$dir\" "
	} else {
	    set dir " "
	}
	puts $f "${header}STANDARD ${mnemo}${dir}${status} [env_elt::get_parameter $key] END STANDARD\n"
	set header ""
    }
    puts $f "END METRICS\n"
    close $f
}
