# This script is dedicated to Logiscope Rule Checker project
# It is used once a Logiscope build is done and once the Logiscope
# API has been instanciated with the data corresponding to the project.
set ::Tool rule

# Name: post_rule
# Parameter: $lang the language of the current project
# Return: None
#
# This proc is called once the Logiscope project is loaded and then
# go through violations in order to transform them in Eclipse objects (violationMarkers).
proc post_rule {lang} {
    set ::Language $lang
    
    # Removes all violations previously set on the project
    eclog::deleteViolations
    
    # Go through Logiscope API
    foreach item $::logiscope {
	MapRole $item project ForEachProject
    }

    if {[eclog::getLinkedState]} {
	# Make a linked file for Logiscope project
	set ttp [eclog::getTtpPath $::Tool]
	if {[file exist $ttp]} {
	    eclog::makeFileLink $::LinkedRuleProjectName $ttp
        } else {
    	    # Remove the linked file for Logiscope project
            eclog::removeFileLink $::LinkedRuleProjectName
        }
	
	# Make a linked file for Logiscope report
	set html [eclog::getReportPath $::Tool]
	if {[file exist $html]} {
	    eclog::makeFileLink $::LinkedRuleReport $html
	} else {
            # Remove a linked file for Logiscope report
            eclog::removeFileLink $::LinkedRuleReport
	}
    }
    return ""
}

# Name: ForEachProject
# Parameter: $project the current Logiscope project
# Return: 1 (to continue processing)
#
# This proc is called on each Logiscope project (normally only one) and 
# call the proc ForEachSourceFile for each source file of the project.
proc ForEachProject {project} {

    MapRole $project sourceFile ForEachSourceFile

    return 1
}

# Name: ForEachSourceFile
# Parameter: $sourceFile the source file to work on
# Return: 1 (to continue processing)
#
# This proc is called for source file of the Logiscope project and
# calls in turn the proc ForEachRuleViolation on each violation for the file $sourceFile
proc ForEachSourceFile {sourceFile} {
    set ::Lst {}
    MapRole $sourceFile ruleViolation "ForEachRuleViolation $sourceFile"
    foreach e $::Lst {
		eclog::addViolation $::Language [lindex $e 0] [lindex $e 1] [lindex $e 2] [lindex $e 3] [lindex $e 4] [lindex $e 5]
    }
    return 1
}

# Name: ForEachRuleViolation
# Parameter: $sourceFile the source file for which violations are detected
#	     $ruleViolation the violation to dump
# Return: 1
#
# This proc writes on the output a violation in the VC++ format in order to allow
# localisation in text editor using double click.
proc ForEachRuleViolation {sourceFile ruleViolation} {
    if {[Get $ruleViolation derogation]} {
        # Relaxed violation, ignore it
        return 1
    }
    
    set rule [GetRole $ruleViolation rule]
    set vbasicRule [GetRole $rule basicRule]
    if { "$vbasicRule" != ""} {
		set key [Get $vbasicRule mnemonic]
		set name [Get $vbasicRule name]
    } else {
		set key [Get $rule name]
		set name "Not available"
    }
    lappend ::Lst [list [Get $sourceFile pathName] [Get $ruleViolation line] [Get $ruleViolation start] [Get $ruleViolation length] $key $name]
    return 1

}
