source [file join [file dirname [info script]] init.tcl] 

source [file join $::CommonDirectory misc.tcl]
source [file join $::EnvironmentDirectory env_log.tcl]
source [file join $::GuiDirectory cmn_gui.tcl]

namespace eval properties {

proc initEnv {language} {
    # Nothing special to initialize when language is all
    if {[string equal $language all]} return

    # Initialize the rule environment
    env_read::init $language
    cmn_gui::setCurrentCfg bidon.cfg 1
    cmn_gui::computeRuleSets
    foreach rs [cmn_gui::getRuleSets] {
        if {[string match "Eclipse (*" $rs]} {
            cmn_gui::setProjectRuleSet [list $rs]
            break
        }
    }
    switch -- $language {
    c - cpp {
        dialects.$language
    }
    }
}

proc setValue {prop {value {}}} {
    set lst [split $prop :]
    set tool [lindex $lst 0]
    set language [lindex $lst 1]
    set name [join [lrange $lst 2 end] :]
    set result ""
    
    switch -regexp -- $name {
    current_rulesets {
        cmn_gui::setProjectRuleSet $value
    }
    ^dialect$ {
        setDialect.$language $value
    }
    }
}

proc computeValue {prop} {
    set lst [split $prop :]
    set tool [lindex $lst 0]
    set language [lindex $lst 1]
    set name [join [lrange $lst 2 end] :]
    set arg  [join [lrange $lst 3 end] :]
    set result ""
    
    switch -regexp -- $name {
    ^directory$ {
        set result ".logiscope_$tool"
    }
    ^relaxation$ {
    	set result 1
    }
    ^frame$ {
    	set result 1
    }
    ^linked$ {
    	set result 1
    }
    ^definition$ {
    	set result ""
    }
    ^ignore$ {
    	set result ""
    }
    ^specific$ {
	# Any language : use or not specific project level settings
	# Default is none.    
    	set result 0
    }
    ^dialects$ {
        # Depends on the language (C or C++)
        set result [dialects.$language]
    }
    ^dialect$ {
        # Depends on the language (C or C++)
        set result [dialect.$language]
    }
    ^dialect_gui:.*$ {
        # Depends on the language (C or C++)
        set result [dialect_gui.$language $arg]
    }
    ^dialect_opt:.*$ {
        # Depends on the language (C or C++)
        set result [dialect_opt.$language $arg]
    }
    ^rulesets$ {
        # Any language
    	set result [rulesets]
    }
    ^current_rulesets$ {
        set result {}
        foreach rs [cmn_gui::getRuleSets] {
            if {[string match "Eclipse (*" $rs]} {
                set result [list $rs]
                break
            }
        }
    }
    ^current_rules$ {
    	cmn_gui::computeRules
        set result [cmn_gui::getRules]
    }
    rule_description:.* {
    	set mnemo $arg
    	set result [cmn_gui::getRuleDescription $mnemo]
    }
    rule_name:.* {
    	set mnemo $arg
        set key [env_elt::get_rule_key $mnemo]
        set result [env_elt::get_name $key]
    }
    rule_state:.* {
    	set mnemo $arg
    	if  {[cmn_gui::getRuleState $mnemo]} {
    	    # By default rules on are seen as warning
    	    set result 1
        } else {
            # By default rules off are seen ignored
            set result 0
        }
    }
    }
    return $result
}

proc rulesets {} {
    set prjrs [cmn_gui::getProjectRuleSet]
    set lst {}
    foreach rs [cmn_gui::getRuleSets] {
        if {[lsearch -exact $prjrs $rs] == -1} {
            lappend lst $rs
        }
    }
    return $lst
}

# Dialect management for C language
proc dialects.c {} {
    set lst [cmn_gui::computeDialects]
    # Sets the default dialect to ansi
    cmn_gui::setDialect ansi
    return $lst
}

proc setDialect.c {value} {
    cmn_gui::setDialect [cmn_gui::getDialect $value]
}

proc dialect.c {} {
    return [cmn_gui::getDialect]
}

proc dialect_gui.c {opt} {
    set save [cmn_gui::getDialect]
    cmn_gui::setDialect $opt
    set result $cmn_gui::DialectName
    cmn_gui::setDialect $save
    return $result
}

proc dialect_opt.c {gui} {
    set save [cmn_gui::getDialect]
    set result [cmn_gui::getDialect $gui]
    cmn_gui::setDialect $save
    return $result    
}

# Dialect (profile) management for C++ language
proc dialects.cpp {} {
    cmn_gui::computeProfiles
    # Strange, but get will set also
    cmn_gui::getProfile C++
    return $cmn_gui::ProfileNames
}

proc setDialect.cpp {value} {
    cmn_gui::setProfile [cmn_gui::getProfile $value]
}

proc dialect.cpp {} {
    if {![info exist cmn_gui::ProfileName]} {
        # Not initialized yet
        dialects.cpp
    }
    return [cmn_gui::getProfile]
}

proc dialect_gui.cpp {opt} {
    set save [cmn_gui::getProfile]
    cmn_gui::setProfile $opt
    set result $cmn_gui::ProfileName
    cmn_gui::setProfile $save
    return $result
}

proc dialect_opt.cpp {gui} {
    set save [cmn_gui::getProfile]
    set result [cmn_gui::getProfile $gui]
    cmn_gui::setProfile $save
    return $result    
}

}

